* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
  OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
  TITLE; ODS TRACE OFF; ODS LISTING; DM 'CLEAR LOG';

***********************************************************************************;
*******   MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR CHAPTER 2 EXAMPLE     *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use ModelR2 macro for chapter 2 cross-sectional models;
* CovFewer =    name of covparms table for baseline empty model;
* CovFewer =    name of covparms table for nested model;
* CovMore =     name of covparms table for comparison model;
%MACRO ModelR2(CovBase=,CovFewer=,CovMore=);
DATA &CovBase.;  LENGTH Name $30.; SET &CovBase.;  Name="&CovBase." ;  
    CALL SYMPUT('EmptyEst',estimate);  RUN;
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&CovMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovBase. &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare;  
    R2_from_Base=(&EmptyEst.-Estimate)/&EmptyEst.;
    R2_Increment=R2_from_Base-LAG1(R2_from_Base); 
    IF _N_=2 THEN R2_Increment=.; RUN;
DATA CovCompare; SET CovCompare; 
    IF CovParm IN("UN(2,1)","UN(3,1)","UN(4,1)","UN(3,2)","UN(4,2)","UN(4,3)") 
    THEN DELETE; RUN;
TITLE9 "R2 (% Reduction) Overall and for &CovFewer. vs. &CovMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE9;
%MEND ModelR2;

***********************************************************************************;
*******             BEGIN DATA MANIPULATION FOR CHAPTER 2 EXAMPLE           *******;
*******                  CHANGE "filesave" to your directory                *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
  %LET filesave= C:\Dropbox\17_CLDP944\CLDP944_Example02a;
* SAS university edition version;
  %LET filesave= /folders/myshortcuts/Dropbox/17_CLDP944/CLDP944_Example02a;
  
* Location for SAS files for these models (uses macro variable filesave);
  LIBNAME filesave "&filesave.";

* Import chapter 2 example data into work library;
DATA work.Chapter2; SET filesave.SAS_Chapter2;
* Center continuous predictors;
age85 = age - 85;
grip9 = grip - 9;
* Creating manual contrasts for dementia groups (to be treated as continuous);
     IF demgroup=1 THEN DO; demNF=0; demNC=0; END; * None group is reference;
ELSE IF demgroup=2 THEN DO; demNF=1; demNC=0; END; * Future group difference;
ELSE IF demgroup=3 THEN DO; demNF=0; demNC=1; END; * Current group difference;
* Labeling new variables;
LABEL   
age85=  "age85: Age in Years (0=85)"
grip9=  "grip9: Grip Strength in Pounds (0=9)"
sexMW=  "sexMW: Sex (0=Men, 1=Women)"
demNF=  "demNF: Dementia Contrast for None=0 vs Future=1"
demNC=  "demNC: Dementia Contrast for None=0 vs Current=1";
RUN;

***********************************************************************************;
*******                       BEGIN EXAMPLE 2a MODELS                       *******;
***********************************************************************************;

* Open output directory to save results to;
ODS RTF FILE="&filesave.\SAS_Example02a_Output.rtf" 
	 STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;
* SAS university edition instead;
ODS RTF FILE="&filesave./SAS_Example02a_Output.rtf" 
	STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE1 "Chapter 2: Descriptive Statistics for Example Variables";
PROC MEANS NDEC=2 DATA=work.Chapter2; VAR age grip cognition; RUN;
PROC FREQ  DATA=work.Chapter2; TABLE sexMW demgroup; RUN;
PROC CORR  DATA=work.Chapter2; VAR age grip sexMW cognition; RUN;
TITLE1;

TITLE1 'Eq 2.3: Empty Means Model';
PROC MIXED DATA=work.Chapter2 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     MODEL cognition =  / SOLUTION DDFM=BW;
     ODS OUTPUT CovParms=CovEmpty;
RUN; TITLE1;
 
TITLE1 'Eq 2.7: Age + Grip + Adding Sex (0=M 1=W, as continuous predictor)';
PROC MIXED DATA=work.Chapter2 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     MODEL cognition = age85 grip9 sexMW / SOLUTION DDFM=BW;
     CONTRAST "Model R2 F-Test with df=3" age85 1, grip9 1, sexMW 1 / CHISQ;
     ODS OUTPUT CovParms=CovAgeGripSex;
RUN; TITLE1;
     * Call macro to calculate R2 for overall model;
       %ModelR2(CovBase=CovEmpty, CovFewer=CovEmpty, CovMore=CovAgeGripSex);

TITLE1 'Eq 2.8: Adding Dementia Group';
TITLE2 'Using Manual Group Contrasts so Reference=None';
TITLE3 'sexMW, demNF, and demNC are all treated as continuous predictors';
PROC MIXED DATA=work.Chapter2 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     MODEL cognition = age85 grip9 sexMW demNF demNC / CHISQ SOLUTION DDFM=BW;
     ODS OUTPUT CovParms=CovDem;
	CONTRAST "Model R2 F-Test with df=5" age85 1, grip9 1, sexmw 1, demNF 1, demNC 1;
     CONTRAST "Omnibus F-Test for Dementia Group with df=2"          demNF 1, demNC 1;
     * Request conditional (adjusted) group means (hold age=85, grip=9, men);
     ESTIMATE "Intercept for None Group"    intercept 1 demNF 0 demNC 0;
     ESTIMATE "Intercept for Future Group"  intercept 1 demNF 1 demNC 0;
     ESTIMATE "Intercept for Current Group" intercept 1 demNF 0 demNC 1;
     * Request group differences (unconditional because there are no interactions);
     ESTIMATE "None   vs. Future Group"   demNF  1 demNC 0;
     ESTIMATE "None   vs. Current Group"  demNF  0 demNC 1;
     ESTIMATE "Future vs. Current Group"  demNF -1 demNC 1;
RUN; TITLE1; TITLE2; TITLE3;
     * Call macro to calculate R2 for overall model and change in R2;
       %ModelR2(CovBase=CovEmpty, CovFewer=CovAgeGripSex, CovMore=CovDem);

TITLE1 'Eq 2.8: Adding Dementia Group';
TITLE2 'Using Manual Group Contrasts so Reference=None';
TITLE3 'sexMW, demNF, and demNC are all treated as continuous predictors';
PROC MIXED DATA=work.Chapter2 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     MODEL cognition = age85 grip9 sexMW demNF demNC / CHISQ SOLUTION DDFM=BW;
     ODS OUTPUT CovParms=CovDem;
	CONTRAST "Model R2 F-Test with df=5" age85 1, grip9 1, sexmw 1, demNF 1, demNC 1;
     CONTRAST "Omnibus F-Test for Dementia Group with df=2"          demNF 1, demNC 1;
     * Request conditional (adjusted) group means (hold age=85, grip=9, men);
     ESTIMATE "Intercept for None Group"    intercept 1 demNF 0 demNC 0; * Given as B0;
     ESTIMATE "Intercept for Future Group"  intercept 1 demNF 1 demNC 0; * Not given (B0+B4); 
     ESTIMATE "Intercept for Current Group" intercept 1 demNF 0 demNC 1; * Not given (B0+B5);
     * Request group differences (unconditional because there are no interactions);
     ESTIMATE "None   vs. Future Group"   demNF  1 demNC 0; * Given as B4;
     ESTIMATE "None   vs. Current Group"  demNF  0 demNC 1; * Given as B5;
     ESTIMATE "Future vs. Current Group"  demNF -1 demNC 1; * Not given (B5-B4);
RUN; TITLE1; TITLE2; TITLE3;

TITLE1 'Eq 2.8: Adding Dementia Group';
TITLE2 'Categorical Predictor for Dementia Group on CLASS statement';
PROC MIXED DATA=work.Chapter2 COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS demgroup; * CLASS statement demgroup replaces previous dem contrasts;
     MODEL cognition = age85 grip9 sexMW demgroup / SOLUTION CHISQ DDFM=BW;
	CONTRAST "Model R2 F-Test with df=5" age85 1, grip9 1, sexmw 1, demgroup -1 1 0, demgroup -1 0 1;
	* Request conditional (adjusted) group means (hold age=85, grip=9, men) and all differences;
     LSMEANS demgroup / DIFF=ALL AT(age85 grip9 sexMW) = (0 0 0);
	* Request conditional (adjusted) group means and all differences;
     LSMEANS demgroup / DIFF=ALL;
*** All of the code below is redundant with LSMEANS, but here is how you get all of the same info;
     CONTRAST "Omnibus F-Test for Dementia Group with df=2"  demgroup -1 1 0, demgroup -1 0 1;
     * Request conditional (adjusted) group means (hold age=85, grip=9, men);
     ESTIMATE "Intercept for None Group"    intercept 1 demgroup 1 0 0; * Not given (B0+B4);
     ESTIMATE "Intercept for Future Group"  intercept 1 demgroup 0 1 0; * Not given (B0+B5);
     ESTIMATE "Intercept for Current Group" intercept 1 demgroup 0 0 1; * Given as B0;
     * Request group differences (unconditional because there are no interactions);
     ESTIMATE "None   vs. Future Group"   demgroup -1 1 0; * Not given (B5-B4);
     ESTIMATE "None   vs. Current Group"  demgroup -1 0 1; * Given as B5;
     ESTIMATE "Future vs. Current Group"  demgroup 0 -1 1; * Given as B4;
RUN; TITLE1; TITLE2;

* Close output;
ODS RTF CLOSE;
